@tailwind base;
@tailwind components;
@tailwind utilities;

:root {
    font-family: Inter;
    font-size: 18px;
    line-height: 24px;
    font-weight: 400;

    font-synthesis: none;
    text-rendering: optimizeLegibility;
    -webkit-font-smoothing: antialiased;
    -moz-osx-font-smoothing: grayscale;
    -webkit-text-size-adjust: 100%;
}